// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Easing.hpp"

using namespace Dawn;

float_t Easing::linear(float_t t) {
  return t;
}

float_t Easing::easeInQuad(float_t t) {
  return t * t;
}

float_t Easing::easeOutQuad(float_t t) {
  return t * (2 - t);
}

float_t Easing::easeInOutQuad(float_t t) {
  return t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
}

float_t Easing::easeInCubic(float_t t) {
  return t * t * t;
}

float_t Easing::easeOutCubic(float_t t) {
  return (--t) * t * t + 1;
}

float_t Easing::easeInOutCubic(float_t t) {
  return t < 0.5 ? 4 * t * t * t : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
}

float_t Easing::easeInQuart(float_t t) {
  return t * t * t * t;
}

float_t Easing::easeOutQuart(float_t t) {
  return 1 - (--t) * t * t * t;
}

float_t Easing::easeInOutQuart(float_t t) {
  return t < 0.5 ? 8 * t * t * t * t : 1 - 8 * (--t) * t * t * t;
}

float_t Easing::easeInQuint(float_t t) {
  return t * t * t * t * t;
}

float_t Easing::easeOutQuint(float_t t) {
  return 1 + (--t) * t * t * t * t;
}

float_t Easing::easeInOutQuint(float_t t) {
  return t < 0.5 ? 16 * t * t * t * t * t : 1 + 16 * (--t) * t * t * t * t;
}

float_t Easing::easeInSine(float_t t) {
  return 1 - cos(t * M_PI_2);
}

float_t Easing::easeOutSine(float_t t) {
  return sin(t * M_PI_2);
}

float_t Easing::easeInOutSine(float_t t) {
  return 0.5 * (1 - cos(M_PI * t));
}

float_t Easing::easeInExpo(float_t t) {
  return t == 0 ? 0 : pow(2, 10 * (t - 1));
}

float_t Easing::easeOutExpo(float_t t) {
  return t == 1 ? 1 : 1 - pow(2, -10 * t);
}

float_t Easing::easeInOutExpo(float_t t) {
  if(t == 0) return 0;
  if(t == 1) return 1;
  if(t < 0.5) return 0.5 * pow(2, 20 * t - 10);
  return 1 - 0.5 * pow(2, -20 * t + 10);
}

float_t Easing::easeInCirc(float_t t) {
  return 1 - sqrt(1 - t * t);
}

float_t Easing::easeOutCirc(float_t t) {
  return sqrt(1 - (--t) * t);
}

float_t Easing::easeInOutCirc(float_t t) {
  return t < 0.5 ? 0.5 * (1 - sqrt(1 - 4 * t * t)) : 0.5 * (sqrt(-((2 * t) - 3) * ((2 * t) - 1)) + 1);
}