// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Player.hpp"
#include "scene/Scene.hpp"
#include "input/InputBinds.hpp"

using namespace Dawn;

void Player::updateCameraPosition() {
  auto c = camera.lock();
  if(!c) return;
  glm::vec3 pos = this->getItem()->getLocalPosition();
  c->getItem()->lookAtPixelPerfect(
    pos + glm::vec3(0, 3, 1),
    pos,
    c->getRenderTarget()->getHeight(),
    c->fov,
    32.0f
  );
}

void Player::onInit() {
  Entity::onInit();

  listeners.push_back(getScene()->onUnpausedUpdate.listen([this](float_t delta){
    if(this->isMoving()) return;

    InputManager &im = getGame()->inputManager;
    bool_t run = im.isDown(InputBind::Run);
    float_t stepSpeed = run ? ENTITY_STEP_SPEED_RUNNING : ENTITY_STEP_SPEED_DEFAULT;

    if(im.isDown(InputBind::Up)) {
      this->move(EntityDirection::Up, stepSpeed);
    } else if(im.isDown(InputBind::Down)) {
      this->move(EntityDirection::Down, stepSpeed);
    } else if(im.isDown(InputBind::Left)) {
      this->move(EntityDirection::Left, stepSpeed);
    } else if(im.isDown(InputBind::Right)) {
      this->move(EntityDirection::Right, stepSpeed);
    } else if(im.isDown(InputBind::Action)) {
      this->interact();
    }
  }));

  listeners.push_back(eventMove.listen([this]() {
    this->updateCameraPosition();
  }));

  this->updateCameraPosition();
}

void Player::interact() {
  // Get direction
  auto tile = entityDirectionGetRelativeTilePosition(
    this->tilePosition, this->direction
  );

  auto at = getMap()->getEntityAt(tile);
  if(at == nullptr) return;

  std::cout << "Interacting" << std::endl;
}

void Player::onDispose() {
  Entity::onDispose();
}