// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "Tile.hpp"

#define CHUNK_WIDTH 32
#define CHUNK_HEIGHT 32
#define CHUNK_DEPTH 8

namespace Dawn {
  struct ChunkPosition {
    int32_t x;
    int32_t y;
    int32_t z;
    
    ChunkPosition operator-(const ChunkPosition &other) const {
      return {
        this->x - other.x,
        this->y - other.y,
        this->z - other.z
      };
    }
  };
  
  class Chunk {
    public:
      struct ChunkPosition position;
      struct Tile tiles[CHUNK_WIDTH][CHUNK_HEIGHT][CHUNK_DEPTH];
      
      Chunk();

      /**
       * Gets the tile at the given position.
       * 
       * @param pos The position of the tile to get.
       * @return The tile at the given position.
       */
      struct Tile getTileAt(const struct TilePosition &pos);

      ~Chunk();
  };
}